import { Component, NgModule, ViewChild, ViewContainerRef, ViewEncapsulation } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { FieldType, FieldWrapper, FormlyModule } from '@ngx-formly/core';
import { CommonModule } from '@angular/common';

class FormlyFieldCheckbox extends FieldType {
    /**
     * @param {?} model
     * @param {?} field
     * @return {?}
     */
    static createControl(model, field) {
        return new FormControl({ value: model ? 'on' : undefined, disabled: field.templateOptions.disabled }, field.validators ? field.validators.validation : undefined, field.asyncValidators ? field.asyncValidators.validation : undefined);
    }
}
FormlyFieldCheckbox.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-checkbox',
                template: `
    <label class="custom-control custom-checkbox">
      <input type="checkbox" [formControl]="formControl"
        *ngIf="!to.hidden" value="on"
        [formlyAttributes]="field" class="custom-control-input">
        {{ to.label }}
        <span class="custom-control-indicator"></span>
    </label>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldCheckbox.ctorParameters = () => [];

class FormlyFieldMultiCheckbox extends FieldType {
    /**
     * @param {?} model
     * @param {?} field
     * @return {?}
     */
    static createControl(model, field) {
        let /** @type {?} */ controlGroupConfig = field.templateOptions.options.reduce((previous, option) => {
            previous[option.key] = new FormControl(model ? model[option.key] : undefined);
            return previous;
        }, {});
        return new FormGroup(controlGroupConfig, field.validators ? field.validators.validation : undefined, field.asyncValidators ? field.asyncValidators.validation : undefined);
    }
}
FormlyFieldMultiCheckbox.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-multicheckbox',
                template: `
    <div *ngFor="let option of to.options" class="checkbox">
      <label class="custom-control custom-checkbox">
        <input type="checkbox" [value]="option.value" [formControl]="formControl.get(option.key)"
        [formlyAttributes]="field" class="custom-control-input">
        {{ option.value }}
        <span class="custom-control-indicator"></span>
      </label>
    </div>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldMultiCheckbox.ctorParameters = () => [];

class FormlyFieldInput extends FieldType {
    /**
     * @return {?}
     */
    get type() {
        return this.to.type || 'text';
    }
}
FormlyFieldInput.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-input',
                template: `
    <input [type]="type" [formControl]="formControl" class="form-control"
      [formlyAttributes]="field" [class.is-invalid]="showError">
    `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldInput.ctorParameters = () => [];

class FormlyFieldRadio extends FieldType {
}
FormlyFieldRadio.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-radio',
                template: `
    <div [formGroup]="form">
      <div *ngFor="let option of to.options" class="radio">
        <label class="custom-control custom-radio">
          <input [name]="id" type="radio" [value]="option.key" [formControl]="formControl"
          [formlyAttributes]="field" class="custom-control-input">
          {{ option.value }}
          <span class="custom-control-indicator"></span>
        </label>
      </div>
    </div>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldRadio.ctorParameters = () => [];

class FormlyFieldTextArea extends FieldType {
}
FormlyFieldTextArea.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-textarea',
                template: `
    <textarea [name]="key" [formControl]="formControl" [cols]="to.cols"
      [rows]="to.rows" class="form-control" [class.is-invalid]="showError"
      [formlyAttributes]="field">
    </textarea>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldTextArea.ctorParameters = () => [];

class FormlyFieldSelect extends FieldType {
    /**
     * @return {?}
     */
    get labelProp() { return this.to.labelProp || 'label'; }
    /**
     * @return {?}
     */
    get valueProp() { return this.to.valueProp || 'value'; }
    /**
     * @return {?}
     */
    get groupProp() { return this.to.groupProp || 'group'; }
    /**
     * @return {?}
     */
    get selectOptions() {
        let /** @type {?} */ options = [];
        this.to.options.map((option) => {
            if (!option[this.groupProp]) {
                options.push(option);
            }
            else {
                let /** @type {?} */ filteredOption = options.filter((filteredOption) => {
                    return filteredOption.label === option[this.groupProp];
                });
                if (filteredOption[0]) {
                    filteredOption[0].group.push({
                        label: option[this.labelProp],
                        value: option[this.valueProp],
                    });
                }
                else {
                    options.push({
                        label: option[this.groupProp],
                        group: [{ value: option[this.valueProp], label: option[this.labelProp] }],
                    });
                }
            }
        });
        return options;
    }
}
FormlyFieldSelect.decorators = [
    { type: Component, args: [{
                selector: 'formly-field-select',
                template: `
    <select [formControl]="formControl" class="form-control" [class.is-invalid]="showError" [formlyAttributes]="field">
      <option value="" *ngIf="to.placeholder">{{ to.placeholder }}</option>
      <ng-container *ngFor="let item of selectOptions">
       <optgroup *ngIf="item.group" label="{{item.label}}">
         <option *ngFor="let child of item.group" [value]="child.value" [disabled]="item.disabled">
           {{ child.label }}
         </option>
       </optgroup>
       <option *ngIf="!item.group" [value]="item.value" [disabled]="item.disabled">{{ item.label }}</option>
      </ng-container>
    </select>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyFieldSelect.ctorParameters = () => [];

class FormlyWrapperFieldset extends FieldWrapper {
}
FormlyWrapperFieldset.decorators = [
    { type: Component, args: [{
                selector: 'formly-wrapper-fieldset',
                template: `
    <div class="form-group">
      <ng-template #fieldComponent></ng-template>
    </div>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyWrapperFieldset.ctorParameters = () => [];
FormlyWrapperFieldset.propDecorators = {
    'fieldComponent': [{ type: ViewChild, args: ['fieldComponent', { read: ViewContainerRef },] },],
};

class FormlyWrapperLabel extends FieldWrapper {
}
FormlyWrapperLabel.decorators = [
    { type: Component, args: [{
                selector: 'formly-wrapper-label',
                template: `
    <label [attr.for]="id" class="form-control-label">
      {{ to.label }}
      {{ to.required ? '*' : '' }}
    </label>
    <ng-template #fieldComponent></ng-template>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyWrapperLabel.ctorParameters = () => [];
FormlyWrapperLabel.propDecorators = {
    'fieldComponent': [{ type: ViewChild, args: ['fieldComponent', { read: ViewContainerRef },] },],
};

class FormlyWrapperDescription extends FieldWrapper {
}
FormlyWrapperDescription.decorators = [
    { type: Component, args: [{
                selector: 'formly-wrapper-description',
                template: `
    <ng-template #fieldComponent></ng-template>
    <div>
      <small class="text-muted">{{ to.description }}</small>
    </div>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyWrapperDescription.ctorParameters = () => [];
FormlyWrapperDescription.propDecorators = {
    'fieldComponent': [{ type: ViewChild, args: ['fieldComponent', { read: ViewContainerRef },] },],
};

class FormlyWrapperValidationMessages extends FieldWrapper {
    /**
     * @return {?}
     */
    get validationId() {
        return this.field.id + '-message';
    }
}
FormlyWrapperValidationMessages.decorators = [
    { type: Component, args: [{
                selector: 'formly-wrapper-validation-messages',
                template: `
    <ng-template #fieldComponent></ng-template>
    <div>
      <small class="invalid-feedback" [style.display]="'block'" *ngIf="showError" role="alert" [id]="validationId">
        <formly-validation-message [fieldForm]="formControl" [field]="field"></formly-validation-message>
      </small>
    </div>
  `,
            },] },
];
/**
 * @nocollapse
 */
FormlyWrapperValidationMessages.ctorParameters = () => [];
FormlyWrapperValidationMessages.propDecorators = {
    'fieldComponent': [{ type: ViewChild, args: ['fieldComponent', { read: ViewContainerRef },] },],
};

class FormlyWrapperAddons extends FieldWrapper {
    /**
     * @param {?} $event
     * @return {?}
     */
    addonRightClick($event) {
        if (this.to.addonRight.onClick) {
            this.to.addonRight.onClick(this.to, this, $event);
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    addonLeftClick($event) {
        if (this.to.addonLeft.onClick) {
            this.to.addonLeft.onClick(this.to, this, $event);
        }
    }
}
FormlyWrapperAddons.decorators = [
    { type: Component, args: [{
                selector: 'formly-wrapper-addons',
                template: `
    <div class="input-group">
      <div class="input-group-addon"
        *ngIf="to.addonLeft"
        [ngStyle]="{cursor: to.addonLeft.onClick ? 'pointer' : 'inherit'}"
        (click)="addonLeftClick($event)">
        <i [ngClass]="to.addonLeft.class" *ngIf="to.addonLeft.class"></i>
        <span *ngIf="to.addonLeft.text">{{ to.addonLeft.text }}</span>
      </div>
      <ng-template #fieldComponent></ng-template>
      <div class="input-group-addon"
        *ngIf="to.addonRight"
        [ngStyle]="{cursor: to.addonRight.onClick ? 'pointer' : 'inherit'}"
        (click)="addonRightClick($event)">
        <i [ngClass]="to.addonRight.class" *ngIf="to.addonRight.class"></i>
        <span *ngIf="to.addonRight.text">{{ to.addonRight.text }}</span>
      </div>
    </div>
  `,
                styles: [`.input-group formly-field-input { display: inline-flex; }`],
                encapsulation: ViewEncapsulation.None,
            },] },
];
/**
 * @nocollapse
 */
FormlyWrapperAddons.ctorParameters = () => [];
FormlyWrapperAddons.propDecorators = {
    'fieldComponent': [{ type: ViewChild, args: ['fieldComponent', { read: ViewContainerRef },] },],
};

class TemplateDescription {
    /**
     * @param {?} fc
     * @return {?}
     */
    run(fc) {
        fc.templateManipulators.postWrapper.push((field) => {
            if (field && field.templateOptions && field.templateOptions.description) {
                return 'description';
            }
        });
    }
}

class TemplateValidation {
    /**
     * @param {?} fc
     * @return {?}
     */
    run(fc) {
        fc.templateManipulators.postWrapper.push((field) => {
            if (field && field.validators) {
                return 'validation-message';
            }
        });
    }
}

class TemplateAddons {
    /**
     * @param {?} fc
     * @return {?}
     */
    run(fc) {
        fc.templateManipulators.postWrapper.push((field) => {
            if (field && field.templateOptions && (field.templateOptions.addonLeft || field.templateOptions.addonRight)) {
                return 'addons';
            }
        });
    }
}

const FIELD_TYPE_COMPONENTS = [
    // types
    FormlyFieldInput,
    FormlyFieldCheckbox,
    FormlyFieldRadio,
    FormlyFieldSelect,
    FormlyFieldTextArea,
    FormlyFieldMultiCheckbox,
    // wrappers
    FormlyWrapperLabel,
    FormlyWrapperDescription,
    FormlyWrapperValidationMessages,
    FormlyWrapperFieldset,
    FormlyWrapperAddons,
];
const BOOTSTRAP_FORMLY_CONFIG = {
    types: [
        {
            name: 'input',
            component: FormlyFieldInput,
            wrappers: ['fieldset', 'label'],
        },
        {
            name: 'checkbox',
            component: FormlyFieldCheckbox,
            wrappers: ['fieldset'],
        },
        {
            name: 'radio',
            component: FormlyFieldRadio,
            wrappers: ['fieldset', 'label'],
        },
        {
            name: 'select',
            component: FormlyFieldSelect,
            wrappers: ['fieldset', 'label'],
        },
        {
            name: 'textarea',
            component: FormlyFieldTextArea,
            wrappers: ['fieldset', 'label'],
            defaultOptions: {
                templateOptions: {
                    cols: 1,
                    rows: 1,
                },
            },
        },
        {
            name: 'multicheckbox',
            component: FormlyFieldMultiCheckbox,
            wrappers: ['fieldset', 'label'],
        },
    ],
    wrappers: [
        { name: 'label', component: FormlyWrapperLabel },
        { name: 'description', component: FormlyWrapperDescription },
        { name: 'validation-message', component: FormlyWrapperValidationMessages },
        { name: 'fieldset', component: FormlyWrapperFieldset },
        { name: 'addons', component: FormlyWrapperAddons },
    ],
    manipulators: [
        { class: TemplateDescription, method: 'run' },
        { class: TemplateValidation, method: 'run' },
        { class: TemplateAddons, method: 'run' },
    ],
};

class FormlyBootstrapModule {
}
FormlyBootstrapModule.decorators = [
    { type: NgModule, args: [{
                declarations: FIELD_TYPE_COMPONENTS,
                imports: [
                    CommonModule,
                    ReactiveFormsModule,
                    FormlyModule.forRoot(BOOTSTRAP_FORMLY_CONFIG),
                ],
            },] },
];
/**
 * @nocollapse
 */
FormlyBootstrapModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { FormlyBootstrapModule, FormlyFieldCheckbox, FormlyFieldMultiCheckbox, FormlyFieldInput, FormlyFieldRadio, FormlyFieldTextArea, FormlyFieldSelect, FormlyWrapperFieldset, FormlyWrapperLabel, FormlyWrapperDescription, FormlyWrapperValidationMessages, TemplateAddons as ɵf, TemplateDescription as ɵd, TemplateValidation as ɵe, BOOTSTRAP_FORMLY_CONFIG as ɵb, FIELD_TYPE_COMPONENTS as ɵa, FormlyWrapperAddons as ɵc };
//# sourceMappingURL=bootstrap.js.map
